@@ -1,7 +1,7 @@
 /* Android activity for Gforth on Android
 
   Authors: Bernd Paysan, Anton Ertl
-  Copyright (C) 2013,2014,2015,2016,2017,2018 Free Software Foundation, Inc.
+  Copyright (C) 2013,2014,2015,2016,2017,2018,2019 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -161,6 +161,7 @@ public class Gforth
 	InputMethodManager mManager;
 	EditorInfo moutAttrs;
 	MyInputConnection mInputConnection;
+	int mcurpos=0, mlen=0;
 	
 	static class MyInputConnection extends BaseInputConnection {
 	    private SpannableStringBuilder mEditable;
@@ -324,14 +325,13 @@ public class Gforth
 	@Override
 	public InputConnection onCreateInputConnection (EditorInfo outAttrs) {
 	    moutAttrs=outAttrs;
-	    outAttrs.inputType = (InputType.TYPE_CLASS_TEXT |
-				  InputType.TYPE_TEXT_FLAG_AUTO_COMPLETE |
+	    outAttrs.inputType = (InputType.TYPE_CLASS_TEXT | /*
+				  InputType.TYPE_TEXT_FLAG_AUTO_COMPLETE | */
 				  InputType.TYPE_TEXT_FLAG_AUTO_CORRECT);
-	    outAttrs.initialSelStart = 1;
-	    outAttrs.initialSelEnd = 1;
+	    outAttrs.initialSelStart = mcurpos;
+	    outAttrs.initialSelEnd = mcurpos+mlen;
 	    outAttrs.packageName = "gnu.gforth";
-	    outAttrs.imeOptions = (EditorInfo.IME_FLAG_NO_FULLSCREEN |
-				   EditorInfo.IME_FLAG_NO_EXTRACT_UI);
+	    outAttrs.imeOptions = (EditorInfo.IME_FLAG_NO_FULLSCREEN);
 	    mInputConnection = new MyInputConnection(this, true);
 	    return mInputConnection;
 	}
@@ -387,6 +387,9 @@ public class Gforth
 	public void hideIME() {
 	    mManager.hideSoftInputFromWindow(getWindowToken(), 0);
 	}
+	public void restartIME() {
+	    mManager.restartInput(this);
+	}
     }
 
     public class RunForth implements Runnable {
@@ -435,6 +438,9 @@ public class Gforth
     public void hideIME() {
 	if(mView!=null) mView.hideIME();
     }
+    public void restartIME() {
+	if(mView!=null) mView.restartIME();
+    }
     public void showStatus() {
 	if (Build.VERSION.SDK_INT < 16) {
 	    getWindow().clearFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
@@ -459,8 +465,11 @@ public class Gforth
     }
     public void setEditLine(String line, int curpos, int len) {
 	// Log.v(TAG, "setEditLine: \"" + line + "\" at: " + curpos + " len: " + len);
-	if(mView!=null && mView.mInputConnection!=null)
+	if(mView!=null && mView.mInputConnection!=null) {
+	    mView.mcurpos = curpos;
+	    mView.mlen = len;
 	    mView.mInputConnection.setEditLine(line, curpos, len);
+	}
     }
 
     @Override
@@ -519,13 +528,19 @@ public class Gforth
 	    notificationManager.createNotificationChannel(notificationChannel);
 	}
 	
-	wl = powerManager.newWakeLock(PowerManager.SCREEN_DIM_WAKE_LOCK |PowerManager.ACQUIRE_CAUSES_WAKEUP |PowerManager.ON_AFTER_RELEASE,"MyLock");
-	wl_cpu = powerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK,"MyCpuLock");
+	wl = powerManager.newWakeLock(PowerManager.SCREEN_DIM_WAKE_LOCK |PowerManager.ACQUIRE_CAUSES_WAKEUP |PowerManager.ON_AFTER_RELEASE,"gnu.gforth:MyLock");
+	wl_cpu = powerManager.newWakeLock(PowerManager.PARTIAL_WAKE_LOCK,"gnu.gforth:MyCpuLock");
 	
 	handler=new Handler();
 	startgps=new Runnable() {
 		public void run() {
-		    locationManager.requestLocationUpdates(args0, argj0, (float)argf0, (LocationListener)gforth);
+		    int permission = PackageManager.PERMISSION_GRANTED;
+		    if (Build.VERSION.SDK_INT >= 23) { // reliably works with Android 6+
+			checkSelfPermission(Manifest.permission.ACCESS_FINE_LOCATION);
+		    }
+		    if (permission == PackageManager.PERMISSION_GRANTED) {
+			locationManager.requestLocationUpdates(args0, argj0, (float)argf0, (LocationListener)gforth);
+		    }
 		}
 	    };
 	stopgps=new Runnable() {
