@@ -1,7 +1,7 @@
 /* Gforth virtual machine (aka inner interpreter)
 
   Authors: Anton Ertl, Bernd Paysan, David Kühling
-  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2010,2011,2012,2013,2014,2015,2016 Free Software Foundation, Inc.
+  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2010,2011,2012,2013,2014,2015,2016,2019 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -35,6 +35,7 @@
 #include <assert.h>
 #include <stdlib.h>
 #include <errno.h>
+#include <signal.h>
 #include "io.h"
 #include "threaded.h"
 #ifndef STANDALONE
@@ -112,11 +113,95 @@ extern Char *gforth_memset(Char * s, Cell c, UCell n);
    switch appropriately, but currently don't.  The CHECK_DIVISION flag
    is for the other cases. */
 #ifdef GFORTH_DEBUGGING
+#if defined(DIVISION_SIGNAL) && defined(SA_SIGINFO)
+/* we know that we get a signal with si_code on division by zero or
+   division overflow */
+#define CHECK_DIVISION_SW 0
+/* we reuse gforth_SP for saving the divisor, as it is not used at the time */
+#define SAVE_DIVISOR(x) do { gforth_SP = (Cell *)(x); \
+    asm volatile("":"+g"(x)::"memory"); } while (0)
+#else /* !(defined(DIVISION_SIGNAL) && defined(SA_SIGINFO)) */
 #define CHECK_DIVISION_SW 1
+#define SAVE_DIVISOR(x) ((void)0)
+#endif /* !(defined(DIVISION_SIGNAL) && defined(SA_SIGINFO)) */
 #define CHECK_DIVISION 1
 #else
 #define CHECK_DIVISION_SW 0
 #define CHECK_DIVISION 0
+#define SAVE_DIVISOR(x) ((void)0)
+#endif
+
+/* buffer for keeping results of stage1 of staged division */
+typedef struct {
+  UCell inverse;    /* (low part of) the inverse */
+  UCell inverse_hi; /* hi part for U/, shift for /F */
+  UCell divisor;    /* for computing the modulus */
+} stagediv_t;
+
+/* Both the function and the macro work, but on Haswell the function is
+   faster because of better register allocation, despite not being inlined */
+#if 1
+static inline UCell uslashstage2(UCell u1, stagediv_t *stage1)
+{
+  UDCell hi=ummul(u1,stage1->inverse_hi);
+  UDCell lo=ummul(u1,stage1->inverse);
+  return DHI(umadd(hi,DHI(lo)));
+}
+#else
+#define uslashstage2(u1,stage1)                                         \
+  DHI(umadd(ummul(u1,stage1->inverse_hi), DHI(ummul(u1,stage1->inverse))))
+#endif
+
+/* using a function because I did not get macros to work correctly; it
+   also happens to have better register allocation on AMD64, at the
+   cost of a function call (and non-relocatability) */
+static inline Cell slashfstage2(Cell n1, stagediv_t *stage1)
+{
+  Cell inv=stage1->inverse;
+  UDCell ud1=D2UD(mmul(n1,inv));
+  UDCell ud2=umadd(ud1,((UCell)inv)>>1);
+  Cell n2 = DHI(ud2)+n1;
+  return n2>>stage1->inverse_hi;
+}
+
+/* these macros did not work */
+/*
+#define slashfstage2(n1,stage1)                                       \
+  (((Cell)(DHI(umadd(D2UD(mmul(n1,stage1->inverse)),stage1->inverse>>1))+n1))>> \
+   stage1->inverse_hi)
+*/
+/*
+#define slashfstage2(_n1,_stage1) \
+  ({                              \
+  Cell n=_n1; \
+  stagediv_t *stage1=(stagediv_t *)_stage1; \
+  Cell inv=stage1->inverse; \
+  UDCell ud1=D2UD(mmul(n,inv)); \
+  UDCell ud2=umadd(ud1,((UCell)inv)>>1); \
+  Cell n2 = DHI(ud2)+n; \
+  n2>>stage1->inverse_hi; \
+  })
+*/
+
+/* ALIVE_DEBUGGING(x) makes x appear to be used (in the debugging
+   engine); we use this in words like DROP to avoid the dead-code
+   elimination of the load of the bottom stack item, in order to get
+   precise stack underflow errors.
+
+   Here's how it works: We first copy x into a variable _x.  The
+   second asm statement makes the compiler believe that it makes use
+   of _x.  The first asm statement makes the compiler believe that the
+   memory location of x may be clobbered, so the compiler actually has
+   to load x into _x if x is a memory reference (e.g., "*lp").
+ */
+#ifdef GFORTH_DEBUGGING
+#define ALIVE_DEBUGGING(x) \
+  do { typeof(x) _x=(x);         \
+    asm volatile("":::"memory"); \
+    asm volatile(""::"X"(_x):"memory");        \
+  } while(0)
+#else
+#define ALIVE_DEBUGGING(x) ((void)0)
 #endif
 
 /* conversion on fetch */
