@@ -1,7 +1,7 @@
 /* common header file
 
   Authors: Bernd Paysan, Anton Ertl, David Kühling, Jens Wilke, Neal Crook
-  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Free Software Foundation, Inc.
+  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -153,13 +153,6 @@ typedef struct {
   Float* fpx;
 } ptrpair;
 
-/* define this false if you want native division */
-#ifdef FORCE_CDIV
-#define FLOORED_DIV 0
-#else
-#define FLOORED_DIV ((1%-3)>0)
-#endif
-
 // prior to 4.8, gcc did not provide __builtin_bswap16 on some platforms so we emulate it
 // see http://gcc.gnu.org/bugzilla/show_bug.cgi?id=52624
 // Clang has a similar problem, but their feature test macros make it easier to detect
@@ -169,18 +162,18 @@ typedef struct {
 # ifdef HAVE___BUILTIN_BSWAP32
 #  define BSWAP16(x) __builtin_bswap32((x) << 16)
 # else
-#  define BSWAP16(x) ((((x)>>8) & 0xff) | ((x)<<8) & 0xff00)
+#  define BSWAP16(x) ((((uint16_t)(x))>>8) | (((uint16_t)(x))<<8))
 # endif
 #endif
 #ifdef HAVE___BUILTIN_BSWAP32
 # define BSWAP32(x) __builtin_bswap32(x)
 #else
-# define BSWAP32(x) ((BSWAP16(x)<<16) | (BSWAP16((x)>>16)))
+# define BSWAP32(x) ((((uint32_t)BSWAP16(x))<<16)|((uint32_t)BSWAP16((x)>>16)))
 #endif
 #ifdef HAVE___BUILTIN_BSWAP64
 # define BSWAP64(x) __builtin_bswap64(x)
 #else
-# define BSWAP64(x) ((BSWAP32(x)<<32) | (BSWAP32((x)>>32)))
+# define BSWAP64(x) ((((uint64_t)BSWAP32(x))<<32)|((uint64_t)BSWAP32((x)>>32)))
 #endif
 
 #if defined(BUGGY_LONG_LONG)
@@ -210,6 +203,8 @@ typedef struct {
 #define DHI_IS(x,y) (x).hi=(y)
 #define DLO_IS(x,y) (x).lo=(y)
 #define D_IS(x,y,z) ({ (x).hi=(y); (x).lo=(z); })
+#define DCELL(x,y)  ((DCell){(x),(y)})
+#define UDCELL(x,y) ((UDCell){(x),(y)})
 
 #define UD2D(ud)	({UDCell _ud=(ud); (DCell){_ud.hi,_ud.lo};})
 #define D2UD(d)		({DCell _d1=(d); (UDCell){_d1.hi,_d1.lo};})
@@ -245,6 +240,8 @@ typedef DOUBLE_UCELL_TYPE UDCell;
 #define DHI_IS(x,y) ({ Double_Store _d; _d.d=(x); _d.cells.high=(y); (x)=_d.d; })
 #define DLO_IS(x,y) ({ Double_Store _d; _d.d=(x); _d.cells.low =(y); (x)=_d.d; })
 #define D_IS(x,y,z) ({ Double_Store _d; _d.cells.high=(y); _d.cells.low =(z); (x)=_d.d; })
+#define DCELL(x,y)  ((Double_Store){.cells={.high=(x),.low=(y)}}.d)
+#define UDCELL(x,y) ((Double_Store){.cells={.high=(x),.low=(y)}}.ud)
 
 #define UD2D(ud)	((DCell)(ud))
 #define D2UD(d)		((UDCell)(d))
@@ -532,6 +529,7 @@ struct Cellpair parse_white(Char *c_addr1, UCell u1);
 Cell rename_file(Char *c_addr1, UCell u1, Char *c_addr2, UCell u2);
 struct Cellquad read_line(Char *c_addr, UCell u1, FILE *wfileid);
 struct Cellpair file_status(Char *c_addr, UCell u);
+struct Cellpair represent(Float r, Address c_addr, UCell u, Cell *np);
 Cell to_float(Char *c_addr, UCell u, Float *r_p, Char dot);
 Float v_star(Float *f_addr1, Cell nstride1, Float *f_addr2, Cell nstride2, UCell ucount);
 void faxpy(Float ra, Float *f_x, Cell nstridex, Float *f_y, Cell nstridey, UCell ucount);
@@ -558,11 +556,23 @@ Sigfunc *bsd_signal(int signo, Sigfunc *func);
 
 /* dblsub routines */
 DCell dnegate(DCell d1);
-UDCell ummul (UCell a, UCell b);
-DCell mmul (Cell a, Cell b);
 UDCell umdiv (UDCell u, UCell v);
 DCell smdiv (DCell num, Cell denom);
 DCell fmdiv (DCell num, Cell denom);
+#ifdef BUGGY_LL_MUL
+UDCell ummul (UCell a, UCell b);
+DCell mmul (Cell a, Cell b);
+#else
+#define ummul(u1,u2) ((UDCell)(u1) * (UDCell)(u2))
+#define mmul(n1,n2) (((DCell)(n1)) * (DCell)(n2))
+#endif
+#ifdef BUGGY_LL_ADD
+UDCell dadd(UDCell x, UDCell y);
+#define umadd(ud,u) dadd(ud, UDCELL(0,u))
+#else
+#define dadd(x,y) (((UDCell)(x))+(UDCell)(y))
+#define umadd(ud,u) (((UDCell)(ud))+(UCell)(u))
+#endif
 
 Cell memcasecmp(const Char *s1, const Char *s2, Cell n);
 
@@ -586,6 +596,7 @@ extern Label *vm_prims;
 extern Label *xts;
 extern Label *labels;
 extern Cell npriminfos;
+extern char gforth_debugging;
 
 #ifdef HAS_DEBUG
 extern int debug, debug_mcheck;
