@@ -1,7 +1,7 @@
 /* header file for libcc-generated C code
 
   Authors: Anton Ertl, Bernd Paysan
-  Copyright (C) 2006,2007,2008,2012,2013,2014,2015,2016,2017 Free Software Foundation, Inc.
+  Copyright (C) 2006,2007,2008,2012,2013,2014,2015,2016,2017,2019 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -164,6 +164,10 @@ gforth_stackpointers gforth_libcc_init(GFORTH_ARGS)
             : 0); \
   } while (0);
 
+#define c_str2gforth_str(str,addr,u) \
+    (addr) = (Cell) str; \
+    (u) = (addr) ? strlen((char*)(addr)) : 0;
+
 #define gforth_ll2ud(ll,lo,hi) \
   do { \
     UClongest _ll = (ll); \
