@@ -2,7 +2,7 @@
 
 
   Authors: Anton Ertl, Bernd Paysan, Jens Wilke, David Kühling
-  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Free Software Foundation, Inc.
+  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -136,6 +136,12 @@ jmp_buf throw_jmp_handler;
 
 #define maxaligned(n)	(typeof(n))((((Cell)n)+sizeof(Float)-1)&-sizeof(Float))
 
+#ifdef GFORTH_DEBUGGING
+char gforth_debugging=1;
+#else
+char gforth_debugging=0;
+#endif
+
 static UCell dictsize=0;
 static UCell dsize=0;
 static UCell rsize=0;
@@ -174,6 +180,7 @@ static int no_super=0;   /* true if compile_prim should not fuse prims */
 static int no_dynamic=NO_DYNAMIC_DEFAULT; /* if true, no code is generated
 					     dynamically */
 static int print_metrics=0; /* if true, print metrics on exit */
+static int print_prims=0; /* if true, print primitives on exit */
 static int static_super_number = 10000; /* number of ss used if available */
 #define MAX_STATE 9 /* maximum number of states */
 static int maxstates = MAX_STATE; /* number of states for stack caching */
@@ -213,6 +220,7 @@ typedef struct {
   Label start; /* NULL if not relocatable */
   Cell length; /* only includes the jump iff superend is true*/
   Cell restlength; /* length of the rest (i.e., the jump or (on superend) 0) */
+  unsigned uses; /* number of uses */
   char superend; /* true if primitive ends superinstruction, i.e.,
                      unconditional branch, execute, etc. */
   Cell nimmargs;
@@ -364,7 +372,7 @@ void gforth_relocate(Address sections[], Char *bitstrings[],
 
     int steps=(((size-1)/sizeof(Cell))/RELINFOBITS)+1;
 
-    debugp(stderr, "relocate section %i, %p:%p\n", ii, base, size);
+    debugp(stderr, "relocate section %i, %p:%lx\n", ii, (void *)base, size);
     
     if(!bitstring) break;
     
@@ -928,6 +936,28 @@ MAYBE_UNUSED static Label bsearch_next(Label key, Label *a, UCell n)
     return bsearch_next(key, a, mid+1);
 }
 
+static int state_map(int state)
+{
+  if (state==0) return STACK_CACHE_DEFAULT;
+  if (state<=STACK_CACHE_DEFAULT) return state-1;
+  return state;
+}
+
+#ifndef NO_DYNAMIC
+static void gforth_printprims()
+{
+  unsigned i;
+  for (i=0; i<npriminfos; i++) {
+    PrimInfo *pi=&priminfos[i];
+    struct cost *sc=&super_costs[i];
+    fprintf(stderr,"%-15s %d-%d %4d %4d %12p len=%3ld rest=%2ld send=%1d\n",
+	    prim_names[i], state_map(sc->state_in), state_map(sc->state_out),
+	    i, pi->uses, pi->start, (long)(pi->length), (long)(pi->restlength),
+	    pi->superend);
+  }
+}
+#endif
+
 static void check_prims(Label symbols1[])
 {
   int i;
@@ -972,8 +1002,9 @@ static void check_prims(Label symbols1[])
     return;
   }
   goto_start = goto_p[0];
-  
+
   priminfos = calloc(i,sizeof(PrimInfo));
+  
   for (i=0; symbols1[i]!=0; i++) {
     int prim_len = ends1[i]-symbols1[i];
     PrimInfo *pi=&priminfos[i];
@@ -987,6 +1018,7 @@ static void check_prims(Label symbols1[])
     pi->superend = superend[i]|no_super;
     pi->length = prim_len;
     pi->restlength = endlabel - symbols1[i] - pi->length;
+    pi->uses = 0;
     pi->nimmargs = 0;
     relocs++;
 #if defined(BURG_FORMAT)
@@ -999,7 +1031,7 @@ static void check_prims(Label symbols1[])
     }
 #else
     debugp(stderr, "%-15s %d-%d %4d %p %p len=%3ld rest=%2ld send=%1d",
-	   prim_names[i], sc->state_in, sc->state_out,
+	   prim_names[i], state_map(sc->state_in), state_map(sc->state_out),
 	   i, s1, s2, (long)(pi->length), (long)(pi->restlength),
 	   pi->superend);
 #endif
@@ -1276,6 +1308,7 @@ static Cell compile_prim_dyn(PrimNum p, Cell *tcp)
         address of generated code for putting it into the threaded code */
 {
   Cell static_prim = (Cell)vm_prims[p];
+
 #if defined(NO_DYNAMIC)
   return static_prim;
 #else /* !defined(NO_DYNAMIC) */
@@ -1283,6 +1316,7 @@ static Cell compile_prim_dyn(PrimNum p, Cell *tcp)
 
   if (no_dynamic)
     return static_prim;
+  priminfos[p].uses++;
   if (p>=npriminfos || !is_relocatable(p)) {
     append_jump();
     return static_prim;
@@ -2233,6 +2267,7 @@ int gforth_args(int argc, char ** argv, char ** path, char ** imagename)
       {"dynamic", no_argument, &no_dynamic, 0},
       {"code-block-size", required_argument, NULL, opt_code_block_size},
       {"print-metrics", no_argument, &print_metrics, 1},
+      {"print-prims", no_argument, &print_prims, 1},
       {"print-sequences", no_argument, &print_sequences, 1},
       {"ss-number", required_argument, NULL, ss_number},
       {"ss-states", required_argument, NULL, ss_states},
@@ -2305,6 +2340,7 @@ Engine Options:\n\
   --offset-image		    Load image at a different position\n\
   -p PATH, --path=PATH		    Search path for finding image and sources\n\
   --print-metrics		    Print some code generation metrics on exit\n\
+  --print-prims			    Print primitives with usage counts on exit\n\
   --print-sequences		    Print primitive sequences for optimization\n\
   -r SIZE, --return-stack-size=SIZE Specify return stack size\n\
   --ss-greedy			    Greedy, not optimal superinst selection\n\
@@ -2417,6 +2453,13 @@ user_area* gforth_stacks(Cell dsize, Cell fsize, Cell rsize, Cell lsize)
   totalsize += 2*SIGSTKSZ;
 #endif
 #ifdef HAVE_MMAP
+#ifdef GFORTH_DEBUGGING
+  /* make sure the stack bottom is page-aligned for stack underflow detection*/
+  dsize = dsizep;
+  rsize = rsizep;
+  fsize = fsizep;
+  lsize = lsizep;
+#endif
   a = (Cell)alloc_mmap(totalsize);
   if (a != (Cell)MAP_FAILED) {
     up0=(user_area*)a; a+=pagesize;
@@ -2610,6 +2653,10 @@ Cell gforth_main(int argc, char **argv, char **env)
   }
   gforth_cleanup();
   gforth_printmetrics();
+#ifndef NO_DYNAMIC
+  if (print_prims)
+    gforth_printprims();
+#endif
   // gforth_free_dict();
 
   return retvalue;
