@@ -1,7 +1,7 @@
 /* Gforth support functions
 
   Authors: Bernd Paysan, Anton Ertl
-  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018 Free Software Foundation, Inc.
+  Copyright (C) 1995,1996,1997,1998,2000,2003,2004,2006,2007,2008,2009,2010,2011,2012,2013,2014,2015,2016,2017,2018,2019 Free Software Foundation, Inc.
 
   This file is part of Gforth.
 
@@ -435,10 +435,10 @@ UCell hashkey2a(Char *s, UCell n)
   int rot1 = w*4-1;
   size_t pagesize=4096; /* a power-of-2 that's at most as large as a page
                            if you change this, change test/other.fs */
-  UCell k0 = w==8 ? 0xb64d532aaaaaaad5 : 0xb653aad5;
+  UCell k0 = w==8 ? 0xb64d532aaaaaaad5LL : 0xb653aad5L;
   /* the 64-bit k0 gives not-so-catastrophic avalanche results on a
      case-insensitive version; the 32-bit version is based on that */
-  UCell upmask = w==8 ? 0x2020202020202020L : 0x20202020L;
+  UCell upmask = w==8 ? 0x2020202020202020LL : 0x20202020L;
   
   UCell h,h1;
   if (n<=w) {
@@ -594,6 +594,65 @@ struct Cellpair file_status(Char *c_addr, UCell u)
   return r;
 }
 
+static void repstr(char * s, char * t, UCell u)
+{
+  UCell slen = strlen(s);
+  if (slen>u)
+    slen = u;
+  memcpy(t,s,slen);
+  memset(t+slen,' ',u-slen);
+}
+
+struct Cellpair represent(Float r, Address c_addr1, UCell u, Cell *np)
+{
+  Cell ok, sign, decpt;
+  struct Cellpair fs;
+  char *c_addr=(char *)c_addr1;
+  char *s;
+  char *t;
+  char buf[u+8]; /* extra chars: .e-9999\0 */
+  if (isnan(r)) {
+    sign = 0;
+    decpt = 0;
+    ok = 0;
+    repstr("NaN",c_addr,u);
+  } else {
+    sign = FLAG(r<0);
+    if (isinf(r)) {
+      decpt=0;
+      ok = 0;
+      if (r<0 && u>0) {
+	*c_addr++ = '-';
+	u--;
+      }
+      repstr("infinity",c_addr,u);
+    } else {
+      ok = -1;
+      r = fabs(r);
+      if(snprintf(buf,u+8,"%.*e",(int)u-1,r)<0)
+	fprintf(stderr,"represent error: %s\n", strerror(errno));
+      for (s=buf, t=c_addr;; s++) {
+	char c = *s;
+	if ('0'<=c && c<='9')
+	  *t++ = c;
+	else if (c != '.')
+	  break;
+      }
+      /* fprintf(stderr,"r=%.*e, t=%p, c_addr=%p, u=%ld\n",(int)u-1,r,t,c_addr,u);*/
+      assert(t == c_addr+u);
+      assert(*s == 'e');
+      s++;
+      if (*s == '+')
+	s++;
+      decpt = atoi(s)+1;
+    }
+  }
+  *np = decpt;
+  fs.n1 = sign;
+  fs.n2 = ok;
+  return fs;
+}
+
 Cell to_float(Char *c_addr, UCell u, Float *rp, Char dot)
 {
   /* convertible string := <significand>[<exponent>]
